# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import mock

from zvmsdk import dist
from zvmsdk import exception
from zvmsdk.tests.unit import base


class RHEL6TestCase(base.SDKTestCase):
    """Testcases for different distros."""

    @classmethod
    def setUpClass(cls):
        super(RHEL6TestCase, cls).setUpClass()
        cls.os_version = 'redhat6.7'

    def setUp(self):
        super(RHEL6TestCase, self).setUp()
        self.dist_manager = dist.LinuxDistManager()
        self.linux_dist = self.dist_manager.get_linux_dist(self.os_version)()

    @mock.patch('zvmsdk.smutclient.SMUTClient.execute_cmd_direct')
    def test_execute_cmd(self, mock_execute_cmd):
        user_id = 'fakevm'
        cmd_str = 'ls'
        mock_execute_cmd.return_value = {'overallRC': 0}
        self.linux_dist.execute_cmd(user_id, cmd_str)
        mock_execute_cmd.assert_called_once_with(user_id, cmd_str)

    def test_create_network_configuration_files(self):
        pass

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_create_mount_point_multipath(self, exec_cmd):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        wwpn = '0x5005076812341234'
        lun = '0x0026000000000000'
        mount_point = '/dev/sde'
        multipath = True
        wwid_raw = 'S: disk/by-id/scsi-36005076801a3813cb800000000000225'
        config_file_lib = '/lib/udev/rules.d/56-zfcp.rules'
        path = '/dev/disk/by-path/ccw-0.0.%(fcp)s-zfcp-%(wwpn)s:%(lun)s'
        srcdev = path % {'fcp': fcp, 'wwpn': wwpn, 'lun': lun}
        check_cmd = 'test -e %s && echo exist' % srcdev
        check_rules_cmd = 'find /lib/udev/rules.d/ -name 56-zfcp.rules'
        exec_cmd.side_effect = [{'response': ['exist']},
                                {'response': [wwid_raw]},
                                {'response': [config_file_lib]},
                                {}, {}, {}]
        query_cmd = ('/sbin/udevadm info --query=all --name=%s | '
                     'grep ^S: | grep scsi-' % srcdev)
        link_item = ('KERNEL==\\"dm-*\\", ENV{DM_UUID}==\\"mpath-'
                     '36005076801a3813cb800000000000225\\", '
                     'SYMLINK+=\\"sde\\"')
        update_rule_cmd = 'echo -e %s >> %s' % (link_item, config_file_lib)
        reload_cmd = 'udevadm control --reload'
        create_symlink_cmd = 'udevadm trigger --sysname-match=dm-*'
        self.linux_dist.create_mount_point(assigner_id, fcp, wwpn, lun,
                                           mount_point, multipath)
        exec_cmd.assert_has_calls([mock.call(assigner_id, check_cmd),
                                   mock.call(assigner_id, query_cmd),
                                   mock.call(assigner_id, check_rules_cmd),
                                   mock.call(assigner_id, update_rule_cmd),
                                   mock.call(assigner_id, reload_cmd),
                                   mock.call(assigner_id, create_symlink_cmd)])

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_create_mount_point_no_multipath(self, exec_cmd):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        wwpn = '0x5005076812341234'
        lun = '0x0026000000000000'
        mount_point = '/dev/sde'
        multipath = False
        wwid_raw = 'S: disk/by-id/scsi-36005076801a3813cb800000000000225'
        config_file_lib = '/lib/udev/rules.d/56-zfcp.rules'
        path = '/dev/disk/by-path/ccw-0.0.%(fcp)s-zfcp-%(wwpn)s:%(lun)s'
        srcdev = path % {'fcp': fcp, 'wwpn': wwpn, 'lun': lun}
        check_cmd = 'test -e %s && echo exist' % srcdev
        check_rules_cmd = 'find /lib/udev/rules.d/ -name 56-zfcp.rules'
        query_cmd = ('/sbin/udevadm info --query=all --name=%s | '
                     'grep ^S: | grep scsi-' % srcdev)
        exec_cmd.side_effect = [{'response': ['exist']},
                                {'response': [wwid_raw]},
                                {'response': [config_file_lib]},
                                {}, {}, {}]
        link_item = ('KERNEL==\\"sd*\\", '
                     'ATTRS{wwpn}==\\"0x5005076812341234\\", '
                     'ATTRS{fcp_lun}==\\"0x0026000000000000\\", '
                     'SYMLINK+=\\"sde%n\\"')
        update_rule_cmd = 'echo -e %s >> %s' % (link_item, config_file_lib)
        reload_cmd = 'udevadm control --reload'
        create_symlink_cmd = 'udevadm trigger --sysname-match=sd*'
        self.linux_dist.create_mount_point(assigner_id, fcp, wwpn, lun,
                                           mount_point, multipath)
        exec_cmd.assert_has_calls([mock.call(assigner_id, check_cmd),
                                   mock.call(assigner_id, query_cmd),
                                   mock.call(assigner_id, check_rules_cmd),
                                   mock.call(assigner_id, update_rule_cmd),
                                   mock.call(assigner_id, reload_cmd),
                                   mock.call(assigner_id, create_symlink_cmd)])

    @mock.patch('zvmsdk.dist.LinuxDist.create_mount_point')
    @mock.patch('zvmsdk.dist.rhel._set_zfcp_multipath')
    @mock.patch('zvmsdk.dist.rhel6._set_zfcp_config_files')
    @mock.patch('zvmsdk.dist.rhel._online_fcp_device')
    @mock.patch('zvmsdk.dist.LinuxDist.check_zfcp_module')
    def test_config_volume_attach_active(self, check_module,
                                         online_device, zfcp_config,
                                         zfcp_multipath, create_mount_point):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        wwpn = '0x5005076812341234'
        lun = '0x0026000000000000'
        multipath = True
        mount_point = '/dev/sdz'
        self.linux_dist.config_volume_attach_active(fcp, assigner_id,
                                                    wwpn, lun, multipath,
                                                    mount_point)
        check_module.assert_called_once_with(assigner_id)
        online_device.assert_called_once_with(assigner_id, fcp)
        zfcp_config.assert_called_once_with(assigner_id, fcp, wwpn, lun)
        zfcp_multipath.assert_called_once_with(assigner_id)
        create_mount_point.assert_called_once_with(assigner_id, fcp, wwpn,
                                                   lun, mount_point,
                                                   multipath)

    @mock.patch('zvmsdk.dist.rhel._offline_fcp_device')
    @mock.patch('zvmsdk.dist.rhel6._set_zfcp_config_files')
    @mock.patch('zvmsdk.dist.rhel._online_fcp_device')
    @mock.patch('zvmsdk.dist.LinuxDist.check_zfcp_module')
    def test_config_volume_attach_active_rollback(self, check_module,
                                                  online_device, zfcp_config,
                                                  offline_device):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        wwpn = '0x5005076812341234'
        lun = '0x0026000000000000'
        multipath = True
        mount_point = '/dev/sdz'
        zfcp_config.side_effect = exception.SDKSMUTRequestFailed({}, 'err')
        self.assertRaises(exception.SDKSMUTRequestFailed,
                          self.linux_dist.config_volume_attach_active,
                          fcp, assigner_id, wwpn, lun, multipath,
                          mount_point)
        check_module.assert_called_once_with(assigner_id)
        online_device.assert_called_once_with(assigner_id, fcp)
        offline_device.assert_called_once_with(assigner_id, fcp, wwpn,
                                               lun, multipath)

    @mock.patch('zvmsdk.dist.LinuxDist.remove_mount_point')
    @mock.patch('zvmsdk.dist.rhel6._restart_multipath')
    @mock.patch('zvmsdk.dist.rhel._offline_fcp_device')
    def test_config_volume_detach_active(self, offline_device,
                                         restart_multipath,
                                         remove_mount_point):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        wwpn = '0x5005076812341234'
        lun = '0x0026000000000000'
        multipath = True
        mount_point = '/dev/sdz'
        self.linux_dist.config_volume_detach_active(fcp, assigner_id,
                                                    wwpn, lun, multipath,
                                                    mount_point)
        offline_device.assert_called_once_with(assigner_id, fcp, wwpn,
                                               lun, multipath)
        restart_multipath.assert_called_once_with(assigner_id)
        remove_mount_point.assert_called_once_with(assigner_id, mount_point,
                                                   multipath)

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_online_fcp_device(self, exec_cmd):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        cmd_cio_ignore = 'cio_ignore -r %s' % fcp
        cmd_online_dev = 'chccwdev -e %s' % fcp
        self.linux_dist._online_fcp_device(assigner_id, fcp)
        exec_cmd.assert_has_calls([mock.call(assigner_id, cmd_cio_ignore),
                                   mock.call(assigner_id, cmd_online_dev)])

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_offline_fcp_device(self, exec_cmd):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        wwpn = '0x5005076812341234'
        lun = '0x0026000000000000'
        multipath = True
        device = '0.0.%s' % fcp
        cmd_offline_dev = 'chccwdev -d %s' % fcp
        data = {'wwpn': wwpn, 'lun': lun,
                'device': device, 'zfcpConf': '/etc/zfcp.conf'}
        cmd_delete_records = ('sed -i -e '
                              '\"/%(device)s %(wwpn)s %(lun)s/d\" '
                              '%(zfcpConf)s' % data)
        self.linux_dist._offline_fcp_device(assigner_id, fcp,
                                            wwpn, lun, multipath)
        exec_cmd.assert_has_calls([mock.call(assigner_id, cmd_offline_dev),
                                   mock.call(assigner_id, cmd_delete_records)])

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_set_zfcp_config_files(self, exec_cmd):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        target_wwpn = '0x5005076812341234'
        target_lun = '0x0026000000000000'
        device = '0.0.%s' % fcp
        # add to port(WWPN)
        unit_add = "echo '%s' > " % target_lun
        unit_add += "/sys/bus/ccw/drivers/zfcp/%(device)s/%(wwpn)s/unit_add"\
                    % {'device': device, 'wwpn': target_wwpn}
        set_zfcp_conf = 'echo %(device)s %(wwpn)s %(lun)s >> /etc/zfcp.conf'\
                        % {'device': device, 'wwpn': target_wwpn,
                           'lun': target_lun}
        trigger_uevent = 'echo add >> /sys/bus/ccw/devices/%s/uevent' % device
        self.linux_dist._set_zfcp_config_files(assigner_id, fcp,
                                               target_wwpn, target_lun)
        exec_cmd.assert_has_calls([mock.call(assigner_id, unit_add),
                                   mock.call(assigner_id, set_zfcp_conf),
                                   mock.call(assigner_id, trigger_uevent)])

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_restart_multipath(self, exec_cmd):
        assigner_id = 'fakeid'
        self.linux_dist._restart_multipath(assigner_id)
        start_multipathd = '/sbin/multipath -r'
        exec_cmd.assert_called_once_with(assigner_id, start_multipathd)

    @mock.patch('zvmsdk.dist.rhel6._restart_multipath')
    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_set_zfcp_multipath(self, exec_cmd, restart_multipath):
        assigner_id = 'fakeid'
        conf_file = '#blacklist {\n'
        conf_file += '#\tdevnode "*"\n'
        conf_file += '#}\n'
        cmd_conf = 'echo -e %s > /etc/multipath.conf' % conf_file
        cmd_mpathconf = 'mpathconf'
        self.linux_dist._set_zfcp_multipath(assigner_id)
        exec_cmd.assert_has_calls([mock.call(assigner_id, cmd_conf),
                                   mock.call(assigner_id, cmd_mpathconf)])
        restart_multipath.assert_called_once_with(assigner_id)


class RHEL7TestCase(base.SDKTestCase):

    @classmethod
    def setUpClass(cls):
        super(RHEL7TestCase, cls).setUpClass()
        cls.os_version = 'redhat7.2'

    def setUp(self):
        super(RHEL7TestCase, self).setUp()
        self.dist_manager = dist.LinuxDistManager()
        self.linux_dist = self.dist_manager.get_linux_dist(self.os_version)()

    @mock.patch('zvmsdk.dist.LinuxDist.create_mount_point')
    @mock.patch('zvmsdk.dist.rhel._set_zfcp_multipath')
    @mock.patch('zvmsdk.dist.rhel7._set_zfcp_config_files')
    @mock.patch('zvmsdk.dist.rhel._online_fcp_device')
    @mock.patch('zvmsdk.dist.LinuxDist.check_zfcp_module')
    def test_config_volume_attach_active(self, check_module,
                                         online_device, zfcp_config,
                                         zfcp_multipath, create_mount_point):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        wwpn = '0x5005076812341234'
        lun = '0x0026000000000000'
        multipath = True
        mount_point = '/dev/sdz'
        self.linux_dist.config_volume_attach_active(fcp, assigner_id,
                                                    wwpn, lun, multipath,
                                                    mount_point)
        check_module.assert_called_once_with(assigner_id)
        online_device.assert_called_once_with(assigner_id, fcp)
        zfcp_config.assert_called_once_with(assigner_id, fcp, wwpn, lun)
        zfcp_multipath.assert_called_once_with(assigner_id)
        create_mount_point.assert_called_once_with(assigner_id, fcp, wwpn,
                                                   lun, mount_point,
                                                   multipath)

    @mock.patch('zvmsdk.dist.rhel._offline_fcp_device')
    @mock.patch('zvmsdk.dist.rhel7._set_zfcp_config_files')
    @mock.patch('zvmsdk.dist.rhel._online_fcp_device')
    @mock.patch('zvmsdk.dist.LinuxDist.check_zfcp_module')
    def test_config_volume_attach_active_rollback(self, check_module,
                                                  online_device, zfcp_config,
                                                  offline_device):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        wwpn = '0x5005076812341234'
        lun = '0x0026000000000000'
        multipath = True
        mount_point = '/dev/sdz'
        zfcp_config.side_effect = exception.SDKSMUTRequestFailed({}, 'err')
        self.assertRaises(exception.SDKSMUTRequestFailed,
                          self.linux_dist.config_volume_attach_active,
                          fcp, assigner_id, wwpn, lun, multipath,
                          mount_point)
        check_module.assert_called_once_with(assigner_id)
        online_device.assert_called_once_with(assigner_id, fcp)
        offline_device.assert_called_once_with(assigner_id, fcp, wwpn,
                                               lun, multipath)

    @mock.patch('zvmsdk.dist.LinuxDist.remove_mount_point')
    @mock.patch('zvmsdk.dist.rhel7._restart_multipath')
    @mock.patch('zvmsdk.dist.rhel._offline_fcp_device')
    def test_config_volume_detach_active(self, offline_device,
                                         restart_multipath,
                                         remove_mount_point):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        wwpn = '0x5005076812341234'
        lun = '0x0026000000000000'
        multipath = True
        mount_point = '/dev/sdz'
        self.linux_dist.config_volume_detach_active(fcp, assigner_id,
                                                    wwpn, lun, multipath,
                                                    mount_point)
        offline_device.assert_called_once_with(assigner_id, fcp, wwpn,
                                               lun, multipath)
        restart_multipath.assert_called_once_with(assigner_id)
        remove_mount_point.assert_called_once_with(assigner_id,
                                                   mount_point,
                                                   multipath)

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_set_zfcp_config_files(self, exec_cmd):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        target_wwpn = '0x5005076812341234'
        target_lun = '0x0026000000000000'
        device = '0.0.%s' % fcp
        # add to port(WWPN)
        unit_add = "echo '%s' > " % target_lun
        unit_add += "/sys/bus/ccw/drivers/zfcp/%(device)s/%(wwpn)s/unit_add"\
                    % {'device': device, 'wwpn': target_wwpn}
        set_zfcp_conf = 'echo %(device)s %(wwpn)s %(lun)s >> /etc/zfcp.conf'\
                        % {'device': device, 'wwpn': target_wwpn,
                           'lun': target_lun}
        self.linux_dist._set_zfcp_config_files(assigner_id, fcp,
                                               target_wwpn, target_lun)
        exec_cmd.assert_has_calls([mock.call(assigner_id, unit_add),
                                   mock.call(assigner_id, set_zfcp_conf)])

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_restart_multipath(self, exec_cmd):
        assigner_id = 'fakeid'
        self.linux_dist._restart_multipath(assigner_id)
        start_multipathd = '/sbin/multipath -r'
        exec_cmd.assert_called_once_with(assigner_id, start_multipathd)

    @mock.patch('zvmsdk.dist.rhel7._restart_multipath')
    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_set_zfcp_multipath(self, exec_cmd, restart_multipath):
        assigner_id = 'fakeid'
        conf_file = '#blacklist {\n'
        conf_file += '#\tdevnode "*"\n'
        conf_file += '#}\n'
        cmd_conf = 'echo -e %s > /etc/multipath.conf' % conf_file
        cmd_mpathconf = 'mpathconf'
        self.linux_dist._set_zfcp_multipath(assigner_id)
        exec_cmd.assert_has_calls([mock.call(assigner_id, cmd_conf),
                                   mock.call(assigner_id, cmd_mpathconf)])
        restart_multipath.assert_called_once_with(assigner_id)


class SLESTestCase(base.SDKTestCase):

    @classmethod
    def setUpClass(cls):
        super(SLESTestCase, cls).setUpClass()

    def setUp(self):
        super(SLESTestCase, self).setUp()
        self.dist_manager = dist.LinuxDistManager()
        self.sles11_dist = self.dist_manager.get_linux_dist('sles11')()
        self.sles12_dist = self.dist_manager.get_linux_dist('sles12')()

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_set_zfcp_config_files(self, exec_cmd):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        target_wwpn = '0x5005076812341234'
        target_lun = '0x0026000000000000'
        device = '0.0.%s' % fcp
        host_config = '/sbin/zfcp_host_configure %s 1' % device
        disk_config = '/sbin/zfcp_disk_configure ' +\
                      '%(device)s %(wwpn)s %(lun)s 1' %\
                      {'device': device, 'wwpn': target_wwpn,
                       'lun': target_lun}
        self.sles11_dist._set_zfcp_config_files(assigner_id, fcp,
                                               target_wwpn, target_lun)
        exec_cmd.assert_has_calls([mock.call(assigner_id, host_config),
                                   mock.call(assigner_id, disk_config)])

        self.sles12_dist._set_zfcp_config_files(assigner_id, fcp,
                                               target_wwpn, target_lun)
        exec_cmd.assert_has_calls([mock.call(assigner_id, host_config),
                                   mock.call(assigner_id, disk_config)])

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_restart_multipath(self, exec_cmd):
        assigner_id = 'fakeid'
        self.sles11_dist._restart_multipath(assigner_id)
        start_multipathd = 'systemctl restart multipathd'
        exec_cmd.assert_called_with(assigner_id, start_multipathd)

        self.sles12_dist._restart_multipath(assigner_id)
        start_multipathd = 'systemctl restart multipathd'
        exec_cmd.assert_called_with(assigner_id, start_multipathd)

    @mock.patch('zvmsdk.dist.sles._restart_multipath')
    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_set_zfcp_multipath(self, exec_cmd, restart_multipath):
        assigner_id = 'fakeid'
        modprobe = 'modprobe dm_multipath'
        conf_file = '#blacklist {\n'
        conf_file += '#\tdevnode "*"\n'
        conf_file += '#}\n'
        cmd_conf = 'echo -e %s > /etc/multipath.conf' % conf_file
        self.sles11_dist._set_zfcp_multipath(assigner_id)
        exec_cmd.assert_has_calls([mock.call(assigner_id, modprobe),
                                   mock.call(assigner_id, cmd_conf)])
        restart_multipath.assert_called_with(assigner_id)

        self.sles12_dist._set_zfcp_multipath(assigner_id)
        exec_cmd.assert_has_calls([mock.call(assigner_id, modprobe),
                                   mock.call(assigner_id, cmd_conf)])
        restart_multipath.assert_called_with(assigner_id)


class UBUNTUTestCase(base.SDKTestCase):

    @classmethod
    def setUpClass(cls):
        super(UBUNTUTestCase, cls).setUpClass()

    def setUp(self):
        super(UBUNTUTestCase, self).setUp()
        self.dist_manager = dist.LinuxDistManager()
        self.linux_dist = self.dist_manager.get_linux_dist('ubuntu16')()

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_check_multipath_tools(self, exec_cmd):
        assigner_id = 'fakeid'
        multipath = 'multipath'
        errmsg = 'multipath-tools not installed.'
        self.linux_dist._check_multipath_tools(assigner_id)
        exec_cmd.assert_called_with(assigner_id, multipath, msg=errmsg)

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_restart_multipath(self, exec_cmd):
        assigner_id = 'fakeid'
        self.linux_dist._restart_multipath(assigner_id)
        reload_map = 'systemctl restart multipath-tools.service'
        exec_cmd.assert_called_with(assigner_id, reload_map)

    @mock.patch('zvmsdk.dist.ubuntu._restart_multipath')
    @mock.patch('zvmsdk.dist.ubuntu._check_multipath_tools')
    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_set_zfcp_multipath(self, exec_cmd, check_multipath,
                                restart_multipath):
        assigner_id = 'fakeid'
        conf_file = '#blacklist {\n'
        conf_file += '#\tdevnode "*"\n'
        conf_file += '#}\n'
        cmd_conf = 'echo -e %s > /etc/multipath.conf' % conf_file
        self.linux_dist._set_zfcp_multipath(assigner_id)
        check_multipath.assert_called_with(assigner_id)
        exec_cmd.assert_called_with(assigner_id, cmd_conf)
        restart_multipath.assert_called_with(assigner_id)

    @mock.patch.object(dist.LinuxDist, 'execute_cmd')
    def test_offline_fcp_device(self, exec_cmd):
        fcp = '1fc5'
        assigner_id = 'fakeid'
        target_wwpn = '0x5005076812341234'
        target_lun = '0x0026000000000000'
        multipath = True
        device = '0.0.%s' % fcp
        target = '%s:%s:%s' % (device, target_wwpn, target_lun)
        disk_offline = '/sbin/chzdev zfcp-lun %s -d' % target
        host_offline = '/sbin/chzdev zfcp-host %s -d' % fcp
        offline_dev = 'chccwdev -d %s' % fcp
        self.linux_dist._offline_fcp_device(assigner_id, fcp,
                                            target_wwpn, target_lun, multipath)
        exec_cmd.assert_has_calls([mock.call(assigner_id, disk_offline),
                                   mock.call(assigner_id, host_offline),
                                   mock.call(assigner_id, offline_dev)])



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os
import six

from zvmsdk import log


LOG = log.LOG


class NotMatch(Exception):
    pass


def make_json(data):
    if not data:
        return {}
    # data = re.sub(r'(\: )%\((.+)\)s([^"])', r'\1"%(int:\2)s"\3', data)
    return json.loads(data)


class APITestBase(object):
    def _compare(self, expected, result):
        matched_value = None

        if expected is None:
            if result is None:
                pass
            elif result == u'':
                pass
            else:
                raise NotMatch('Expected None, got %(result)s.'
                        % {'result': result})
        elif isinstance(expected, dict):
            if not isinstance(result, dict):
                raise NotMatch('%(result)s is not a dict.'
                        % {'result': result})
            ex_keys = sorted(expected.keys())
            res_keys = sorted(result.keys())
            if ex_keys != res_keys:
                ex_delta = []
                res_delta = []
                for key in ex_keys:
                    if key not in res_keys:
                        ex_delta.append(key)
                for key in res_keys:
                    if key not in ex_keys:
                        res_delta.append(key)
                raise NotMatch(
                        'Dictionary key mismatch:\n'
                        'Extra key(s) in template:\n%(ex_delta)s\n'
                        'Extra key(s) in result:\n%(res_delta)s\n' %
                        {'ex_delta': ex_delta, 'res_delta': res_delta})
            for key in ex_keys:
                res = self._compare(expected[key], result[key])
                matched_value = res or matched_value
        elif isinstance(expected, list):
            pass
        elif isinstance(expected, (six.integer_types, float,
                                   six.string_types)):
            pass

        return matched_value

    def _read_template(self, name):
        dirname = os.path.dirname(os.path.abspath(__file__))
        dirname = os.path.normpath(os.path.join(dirname,
                                   './api_templates'))

        template = ''.join([dirname, '/', name, '.tpl'])
        with open(template) as inf:
            return inf.read().strip()

    def _verify_result(self, expected, result):

        try:
            expected_json = make_json(expected)
            result_json = make_json(result)

            self._compare(expected_json, result_json)
        except NotMatch:
            raise NotMatch("\nFailed to match Template to Response: \n"
                           "Template: %s\n"
                           "Result: %s\n" %
                           (expected_json, result_json))

    def verify_result(self, test_name, result):
        result = bytes.decode(result)
        expected = self._read_template(test_name)
        return self._verify_result(expected, result)



# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Deployment handling for sdk API."""

import json
import six
import sys
import traceback
import webob

from zvmsdk import log
from zvmsdk.sdkwsgi import handler
from zvmsdk.sdkwsgi import requestlog
from zvmsdk.sdkwsgi import util


LOG = log.LOG
NAME = "zvm-cloud-connector"


def _find_fault(clazz, encountered=None):
    if not encountered:
        encountered = []
    for subclass in clazz.__subclasses__():
        if subclass not in encountered:
            encountered.append(subclass)
            for subsubclass in _find_fault(subclass, encountered):
                yield subsubclass
            yield subclass


class Fault(webob.exc.HTTPException):

    def __init__(self, exception):
        self.wrapped_exc = exception
        for key, value in list(self.wrapped_exc.headers.items()):
            self.wrapped_exc.headers[key] = str(value)
        self.status_int = exception.status_int

    @webob.dec.wsgify()
    def __call__(self, req):

        code = self.wrapped_exc.status_int
        explanation = self.wrapped_exc.explanation
        LOG.debug("Returning %(code)s to user: %(explanation)s",
                  {'code': code, 'explanation': explanation})

        fault_data = {
                'overallRC': 400,
                'rc': 400,
                'rs': code,
                'modID': util.SDKWSGI_MODID,
                'output': '',
                'errmsg': explanation}
        if code == 413 or code == 429:
            retry = self.wrapped_exc.headers.get('Retry-After', None)
            if retry:
                fault_data['retryAfter'] = retry

        self.wrapped_exc.content_type = 'application/json'
        self.wrapped_exc.charset = 'UTF-8'
        self.wrapped_exc.text = six.text_type(json.dumps(fault_data))

        return self.wrapped_exc

    def __str__(self):
        return self.wrapped_exc.__str__()


class FaultWrapper(object):
    """Calls down the middleware stack, making exceptions into faults."""

    _status_to_type = {}

    @staticmethod
    def status_to_type(status):
        if not FaultWrapper._status_to_type:
            for clazz in _find_fault(webob.exc.HTTPError):
                FaultWrapper._status_to_type[clazz.code] = clazz
        return FaultWrapper._status_to_type.get(
                                  status, webob.exc.HTTPInternalServerError)()

    def __init__(self, application):
        self.application = application

    def _error(self, inner, req):
        exc_info = traceback.extract_tb(sys.exc_info()[2])[-1]
        LOG.info('Got unhandled exception: %s', exc_info)

        safe = getattr(inner, 'safe', False)
        headers = getattr(inner, 'headers', None)
        status = getattr(inner, 'code', 500)
        if status is None:
            status = 500

        outer = self.status_to_type(status)
        if headers:
            outer.headers = headers

        if safe:
            outer.explanation = '%s: %s' % (inner.__class__.__name__,
                                            inner.message)

        return Fault(outer)

    @webob.dec.wsgify()
    def __call__(self, req):
        try:
            return req.get_response(self.application)
        except Exception as ex:
            return self._error(ex, req)


class HeaderControl(object):

    def __init__(self, application):
        self.application = application

    @webob.dec.wsgify
    def __call__(self, req):

        response = req.get_response(self.application)
        response.headers.add('cache-control', 'no-cache')
        return response


def deploy(project_name):
    """Assemble the middleware pipeline"""
    request_log = requestlog.RequestLog
    header_addon = HeaderControl
    fault_wrapper = FaultWrapper
    application = handler.SdkHandler()

    # currently we have 3 middleware
    for middleware in (header_addon,
                       fault_wrapper,
                       request_log,
                       ):
        if middleware:
            application = middleware(application)

    return application


def loadapp(project_name=NAME):
    application = deploy(project_name)
    return application


def init_application():
    # build and return WSGI app
    return loadapp()



# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import threading
import time

from zvmsdk import config
from zvmsdk import log
from zvmsdk import smutclient
from zvmsdk import utils as zvmutils

_MONITOR = None
CONF = config.CONF
LOG = log.LOG


def get_monitor():
    global _MONITOR
    if _MONITOR is None:
        _MONITOR = ZVMMonitor()
    return _MONITOR


class ZVMMonitor(object):
    """Monitor support for ZVM"""
    _TYPES = ('cpumem', 'vnics')

    def __init__(self):
        self._cache = MeteringCache(self._TYPES)
        self._smutclient = smutclient.get_smutclient()
        self._namelist = zvmutils.get_namelist()

    def inspect_stats(self, uid_list):
        cpumem_data = self._get_inspect_data('cpumem', uid_list)
        # construct and return final result
        stats_data = {}
        for uid in uid_list:
            if uid in cpumem_data:
                with zvmutils.expect_invalid_resp_data():
                    user_data = cpumem_data[uid]
                    guest_cpus = int(user_data['guest_cpus'])
                    used_cpu_time = user_data['used_cpu_time']
                    used_cpu_time = int(used_cpu_time.partition(' ')[0])
                    elapsed_cpu_time = int(
                        user_data['elapsed_cpu_time'].partition(' ')[0])
                    used_mem = int(user_data['used_memory'].partition(' ')[0])
                    max_mem = int(user_data['max_memory'].partition(' ')[0])
                    min_mem = int(user_data['min_memory'].partition(' ')[0])
                    shared_mem = int(
                        user_data['shared_memory'].partition(' ')[0])

                stats_data[uid] = {
                    'guest_cpus': guest_cpus,
                    'used_cpu_time_us': used_cpu_time,
                    'elapsed_cpu_time_us': elapsed_cpu_time,
                    'min_cpu_count': int(user_data['min_cpu_count']),
                    'max_cpu_limit': int(user_data['max_cpu_limit']),
                    'samples_cpu_in_use': int(user_data['samples_cpu_in_use']),
                    'samples_cpu_delay': int(user_data['samples_cpu_delay']),
                    'used_mem_kb': used_mem,
                    'max_mem_kb': max_mem,
                    'min_mem_kb': min_mem,
                    'shared_mem_kb': shared_mem
                    }

        return stats_data

    def inspect_vnics(self, uid_list):
        vnics = self._get_inspect_data('vnics', uid_list)
        # construct and return final result
        target_vnics = {}
        for uid in uid_list:
            if uid in vnics:
                with zvmutils.expect_invalid_resp_data():
                    target_vnics[uid] = vnics[uid]

        return target_vnics

    def _cache_enabled(self):
        return CONF.monitor.cache_interval > 0

    def _get_inspect_data(self, type, uid_list):
        inspect_data = {}
        update_needed = False
        for uid in uid_list:
            if not zvmutils.valid_userid(uid):
                continue
            cache_data = self._cache.get(type, uid)
            if cache_data is not None:
                inspect_data[uid] = cache_data
            else:
                if self._smutclient.get_power_state(uid) == 'on':
                    update_needed = True
                    inspect_data = {}
                    break

        # If all data are found in cache, just return
        if not update_needed:
            return inspect_data

        # Call client to query latest data
        rdata = {}
        if type == 'cpumem':
            rdata = self._update_cpumem_data(uid_list)
        elif type == 'vnics':
            rdata = self._update_nic_data()

        return rdata

    def _update_cpumem_data(self, uid_list):

        namelist_uids = self._smutclient.namelist_query(self._namelist)
        sdk_managed_uids = self._smutclient.get_vm_list()
        mis_uids = list((set(uid_list) -
                     set(namelist_uids)).intersection(set(sdk_managed_uids)))

        for muid in mis_uids:
            self._smutclient.namelist_add(self._namelist, muid)

        rdata = {}
        if self._cache_enabled():
            rdata = self._smutclient.system_image_performance_query(
                self._namelist)
            self._cache.refresh('cpumem', rdata)
        else:
            rdata = self._smutclient.system_image_performance_query(
                self._namelist)

        return rdata

    def _update_nic_data(self):
        nics = {}
        vsw_dict = self._smutclient.virtual_network_vswitch_query_byte_stats()
        with zvmutils.expect_invalid_resp_data():
            for vsw in vsw_dict['vswitches']:
                for nic in vsw['nics']:
                    userid = nic['userid']
                    nic_entry = {
                        'vswitch_name': vsw['vswitch_name'],
                        'nic_vdev': nic['vdev'],
                        'nic_fr_rx': int(nic['nic_fr_rx']),
                        'nic_fr_tx': int(nic['nic_fr_tx']),
                        'nic_fr_rx_dsc': int(nic['nic_fr_rx_dsc']),
                        'nic_fr_tx_dsc': int(nic['nic_fr_tx_dsc']),
                        'nic_fr_rx_err': int(nic['nic_fr_rx_err']),
                        'nic_fr_tx_err': int(nic['nic_fr_tx_err']),
                        'nic_rx': int(nic['nic_rx']),
                        'nic_tx': int(nic['nic_tx'])}
                    if nics.get(userid, None) is None:
                        nics[userid] = [nic_entry]
                    else:
                        nics[userid].append(nic_entry)
        # Update cache if enabled
        if self._cache_enabled():
            self._cache.refresh('vnics', nics)

        return nics


class MeteringCache(object):
    """Cache for metering data."""

    def __init__(self, types):
        self._cache = {}
        self._types = types
        self._lock = threading.RLock()
        self._reset(types)

    def _reset(self, types):
        with zvmutils.acquire_lock(self._lock):
            for type in types:
                self._cache[type] = {'expiration': time.time(),
                                    'data': {},
                                    }

    def _get_ctype_cache(self, ctype):
        return self._cache[ctype]

    def set(self, ctype, key, data):
        """Set or update cache content.

        :param ctype: cache type
        :param key: the key to be set value
        :param data: cache data
        """
        with zvmutils.acquire_lock(self._lock):
            target_cache = self._get_ctype_cache(ctype)
            target_cache['data'][key] = data

    def get(self, ctype, key):
        with zvmutils.acquire_lock(self._lock):
            target_cache = self._get_ctype_cache(ctype)
            if(time.time() > target_cache['expiration']):
                return None
            else:
                return target_cache['data'].get(key, None)

    def delete(self, ctype, key):
        with zvmutils.acquire_lock(self._lock):
            target_cache = self._get_ctype_cache(ctype)
            if key in target_cache['data']:
                del target_cache['data'][key]

    def clear(self, ctype='all'):
        with zvmutils.acquire_lock(self._lock):
            if ctype == 'all':
                self._reset()
            else:
                target_cache = self._get_ctype_cache(ctype)
                target_cache['data'] = {}

    def refresh(self, ctype, data):
        with zvmutils.acquire_lock(self._lock):
            self.clear(ctype)
            target_cache = self._get_ctype_cache(ctype)
            target_cache['expiration'] = (time.time() +
                                            float(CONF.monitor.cache_interval))
            for (k, v) in data.items():
                self.set(ctype, k, v)



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import jwt
import mock
import unittest

from zvmsdk import exception
from zvmsdk import config
from zvmsdk.sdkwsgi.handlers import vswitch
from zvmsdk.sdkwsgi import util


CONF = config.CONF

FAKE_UUID = '00000000-0000-0000-0000-000000000000'


def set_conf(section, opt, value):
    CONF[section][opt] = value


class FakeResp(object):
    def __init__(self):
        self.body = {}


class FakeReq(object):
    def __init__(self):
        self.headers = {}
        self.environ = {}
        self.__name__ = ''
        self.response = FakeResp()

    def __getitem__(self, name):
        return self.headers


class HandlersGuestTest(unittest.TestCase):

    def setUp(self):
        set_conf('wsgi', 'auth', 'none')
        expired_elapse = datetime.timedelta(seconds=100)
        expired_time = datetime.datetime.utcnow() + expired_elapse
        payload = jwt.encode({'exp': expired_time}, 'username')

        self.req = FakeReq()
        self.req.headers['X-Auth-Token'] = payload

    @mock.patch.object(vswitch.VswitchAction, 'list')
    def test_vswitch_list(self, mock_list):

        mock_list.return_value = ''
        vswitch.vswitch_list(self.req)
        self.assertTrue(mock_list.called)

    @mock.patch.object(vswitch.VswitchAction, 'create')
    def test_vswitch_create(self, mock_create):
        mock_create.return_value = {}
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234 abcd 123F",
                                   "port_type": 1,
                                   "controller": "*"}}"""
        self.req.body = body_str

        vswitch.vswitch_create(self.req)
        body = util.extract_json(body_str)
        mock_create.assert_called_once_with(body=body)

    @mock.patch.object(vswitch.VswitchAction, 'create')
    def test_vswitch_create_with_userid_controller(self, mock_create):
        mock_create.return_value = {}
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234 abcd 123F",
                                   "port_type": 1,
                                   "controller": "userid01"}}"""
        self.req.body = body_str

        vswitch.vswitch_create(self.req)
        body = util.extract_json(body_str)
        mock_create.assert_called_once_with(body=body)

    def test_vswitch_create_invalidname(self):
        body_str = '{"vswitch": {"name": "", "rdev": "1234"}}'
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    def test_vswitch_create_invalid_rdevlist(self):
        body_str = '{"vswitch": {"name": "name1", "rdev": "12345 sss"}}'
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    @mock.patch.object(util, 'wsgi_path_item')
    @mock.patch.object(vswitch.VswitchAction, 'delete')
    def test_vswitch_delete(self, mock_delete, mock_name):
        mock_delete.return_value = {}
        mock_name.return_value = 'vsw1'

        vswitch.vswitch_delete(self.req)
        mock_delete.assert_called_once_with('vsw1')

    @mock.patch.object(util, 'wsgi_path_item')
    @mock.patch.object(vswitch.VswitchAction, 'query')
    def test_vswitch_query(self, mock_query, mock_name):
        mock_query.return_value = {}
        mock_name.return_value = 'vsw1'

        vswitch.vswitch_query(self.req)
        mock_query.assert_called_once_with('vsw1')

    def test_vswitch_create_invalid_connection(self):
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234",
                                   "connection": 3}}"""
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    def test_vswitch_create_invalid_queue_mem(self):
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234",
                                   "queue_mem": 10}}"""
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    def test_vswitch_create_invalid_network_type(self):
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234",
                                   "network_type": 3}}"""
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    def test_vswitch_create_invalid_update(self):
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234",
                                   "update": 4}}"""
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    def test_vswitch_create_invalid_vid(self):
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234",
                                   "vid": -1}}"""
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    def test_vswitch_create_invalid_native_vid(self):
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234",
                                   "native_vid": 4096}}"""
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    def test_vswitch_create_invalid_router(self):
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234",
                                   "router": 3}}"""
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    def test_vswitch_create_invalid_grvp(self):
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234",
                                   "gvrp": 3}}"""
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    def test_vswitch_create_invalid_controller(self):
        body_str = """{"vswitch": {"name": "name1",
                                   "rdev": "1234",
                                   "controller": "node12345"}}"""
        self.req.body = body_str

        self.assertRaises(exception.ValidationError, vswitch.vswitch_create,
                          self.req)

    @mock.patch.object(util, 'wsgi_path_item')
    @mock.patch.object(vswitch.VswitchAction, 'update')
    def test_vswitch_update(self, mock_update, mock_name):
        mock_name.return_value = 'vsw1'
        body_str = '{"vswitch": {"grant_userid": "user1"}}'
        mock_update.return_value = {}
        self.req.body = body_str

        vswitch.vswitch_update(self.req)
        body = util.extract_json(body_str)
        mock_update.assert_called_once_with('vsw1', body=body)



# Copyright 2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from smutLayer import makeVM
from smutLayer import ReqHandle
from smutLayer.tests.unit import base


class SMUTMakeVMTestCase(base.SMUTTestCase):
    """Test cases for makeVM.py in smutLayer."""

    def test_getReservedMemSize(self):
        rh = mock.Mock()
        rh.results = {'overallRC': 0, 'rc': 0, 'rs': 0}
        gap = makeVM.getReservedMemSize(rh, '1024M', '128g')
        self.assertEqual(gap, '130048M')

    def test_getReservedMemSize_invalid_suffix(self):
        rh = ReqHandle.ReqHandle(captureLogs=False,
                                 smut=mock.Mock())
        gap = makeVM.getReservedMemSize(rh, '1024M', '128T')
        self.assertEqual(gap, '0M')
        self.assertEqual(rh.results['overallRC'], 4)
        self.assertEqual(rh.results['rc'], 4)
        self.assertEqual(rh.results['rs'], 205)

    def test_getReservedMemSize_max_less_than_initial(self):
        rh = ReqHandle.ReqHandle(captureLogs=False,
                                 smut=mock.Mock())
        gap = makeVM.getReservedMemSize(rh, '64G', '32G')
        self.assertEqual(gap, '0M')
        self.assertEqual(rh.results['overallRC'], 4)
        self.assertEqual(rh.results['rc'], 4)
        self.assertEqual(rh.results['rs'], 206)

    def test_getReservedMemSize_equal_size(self):
        rh = ReqHandle.ReqHandle(captureLogs=False,
                                 smut=mock.Mock())
        gap = makeVM.getReservedMemSize(rh, '1024M', '1G')
        self.assertEqual(gap, '0M')
        self.assertEqual(rh.results['overallRC'], 0)

    def test_getReservedMemSize_gap_G(self):
        rh = ReqHandle.ReqHandle(captureLogs=False,
                                 smut=mock.Mock())
        gap = makeVM.getReservedMemSize(rh, '512m', '9999G')
        self.assertEqual(gap, '9998G')
        self.assertEqual(rh.results['overallRC'], 0)



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import shutil

from zvmsdk.tests.unit import base
from zvmsdk import dist
from zvmsdk import networkops


class SDKNetworkOpsTestCase(base.SDKTestCase):

    def setUp(self):
        self.networkops = networkops.get_networkops()

    @mock.patch('zvmsdk.smutclient.SMUTClient.create_nic')
    def test_create_nic(self, create_nic):
        self.networkops.create_nic("fakeid", '1000', 'Fake_nic_id',
                                   active=True)
        create_nic.assert_called_with("fakeid", vdev='1000',
                                      nic_id='Fake_nic_id',
                                      mac_addr=None, active=True)

    @mock.patch('zvmsdk.smutclient.SMUTClient.get_vswitch_list')
    def test_get_vswitch_list(self, get_vswitch_list):
        self.networkops.get_vswitch_list()
        get_vswitch_list.assert_called_with()

    @mock.patch('zvmsdk.smutclient.SMUTClient.couple_nic_to_vswitch')
    def test_couple_nic_to_vswitch(self, couple_nic_to_vswitch):
        self.networkops.couple_nic_to_vswitch("fake_userid", "nic_vdev",
                                              "fake_VS_name",
                                              True)
        couple_nic_to_vswitch.assert_called_with("fake_userid",
                                                 "nic_vdev",
                                                 "fake_VS_name",
                                                 active=True)

    @mock.patch('zvmsdk.smutclient.SMUTClient.uncouple_nic_from_vswitch')
    def test_uncouple_nic_from_vswitch(self, uncouple_nic_from_vswitch):
        self.networkops.uncouple_nic_from_vswitch("fake_userid",
                                                  "nic_vdev",
                                                  True)
        uncouple_nic_from_vswitch.assert_called_with("fake_userid",
                                                     "nic_vdev",
                                                     active=True)

    @mock.patch('zvmsdk.smutclient.SMUTClient.add_vswitch')
    def test_add_vswitch(self, add_vswitch):
        self.networkops.add_vswitch("fakename", "fakerdev",
                                    controller='*',
                                    connection='CONNECT',
                                    network_type='ETHERNET',
                                    router="NONROUTER", vid='UNAWARE',
                                    port_type='ACCESS', gvrp='GVRP',
                                    queue_mem=8, native_vid=2, persist=False)
        add_vswitch.assert_called_with("fakename", rdev="fakerdev",
                                       controller='*', connection='CONNECT',
                                       network_type='ETHERNET',
                                       router="NONROUTER",
                                       vid='UNAWARE', port_type='ACCESS',
                                       gvrp='GVRP', queue_mem=8,
                                       native_vid=2, persist=False)

    @mock.patch('zvmsdk.smutclient.SMUTClient.grant_user_to_vswitch')
    def test_grant_user_to_vswitch(self, grant_user):
        self.networkops.grant_user_to_vswitch("vswitch_name", "userid")
        grant_user.assert_called_with("vswitch_name", "userid")

    @mock.patch('zvmsdk.smutclient.SMUTClient.revoke_user_from_vswitch')
    def test_revoke_user_from_vswitch(self, revoke_user):
        self.networkops.revoke_user_from_vswitch("vswitch_name", "userid")
        revoke_user.assert_called_with("vswitch_name", "userid")

    @mock.patch('zvmsdk.smutclient.SMUTClient.set_vswitch_port_vlan_id')
    def test_set_vswitch_port_vlan_id(self, set_vswitch):
        self.networkops.set_vswitch_port_vlan_id("vswitch_name",
                                                 "userid", "vlan_id")
        set_vswitch.assert_called_with("vswitch_name", "userid", "vlan_id")

    @mock.patch('zvmsdk.smutclient.SMUTClient.set_vswitch')
    def test_set_vswitch(self, set_vswitch):
        self.networkops.set_vswitch("vswitch_name", grant_userid='fake_id')
        set_vswitch.assert_called_with("vswitch_name", grant_userid='fake_id')

    @mock.patch('zvmsdk.smutclient.SMUTClient.delete_vswitch')
    def test_delete_vswitch(self, delete_vswitch):
        self.networkops.delete_vswitch("vswitch_name", True)
        delete_vswitch.assert_called_with("vswitch_name", True)

    @mock.patch('zvmsdk.smutclient.SMUTClient.delete_nic')
    def test_delete_nic(self, delete_nic):
        self.networkops.delete_nic("userid", "vdev", True)
        delete_nic.assert_called_with("userid", "vdev",
                                      active=True)

    @mock.patch('zvmsdk.smutclient.SMUTClient.get_nic_info')
    def test_get_nic_info(self, get_nic_info):
        self.networkops.get_nic_info(userid='testid', vswitch='VSWITCH')
        get_nic_info.assert_called_with(userid='testid', nic_id=None,
                                        vswitch='VSWITCH')

    @mock.patch.object(shutil, 'rmtree')
    @mock.patch('zvmsdk.smutclient.SMUTClient.execute_cmd')
    @mock.patch('zvmsdk.smutclient.SMUTClient.update_guestdb_with_net_set')
    @mock.patch('zvmsdk.smutclient.SMUTClient.punch_file')
    @mock.patch('zvmsdk.networkops.NetworkOPS._generate_network_doscript')
    @mock.patch('zvmsdk.smutclient.SMUTClient.is_first_network_config')
    @mock.patch('zvmsdk.smutclient.SMUTClient.get_guest_temp_path')
    def test_network_configuration(self, temp_path, is_first, doscript, punch,
                                   update_guestdb, execute_cmd, rmtree):
        userid = 'fakeid'
        os_version = 'rhel7.2'
        network_info = []
        network_file_path = '/tmp'
        active_cmds = 'execute command'
        network_doscript = 'file'

        temp_path.return_value = network_file_path
        is_first.return_value = True
        doscript.return_value = (network_doscript, active_cmds)
        rmtree.return_value = None

        self.networkops.network_configuration(userid, os_version, network_info,
                              active=True)
        temp_path.assert_called_with(userid)
        is_first.assert_called_with(userid)
        doscript.assert_called_with(userid, os_version, network_info,
                                    network_file_path, True, active=True)
        punch.assert_called_with(userid, network_doscript, "X")
        update_guestdb.assert_called_with(userid)
        execute_cmd.assert_called_with(userid, active_cmds)

    @mock.patch('zvmsdk.dist.LinuxDistManager.get_linux_dist')
    @mock.patch.object(dist.rhel7, 'create_network_configuration_files')
    @mock.patch('zvmsdk.networkops.NetworkOPS._create_znetconfig')
    @mock.patch('zvmsdk.networkops.NetworkOPS._add_file')
    @mock.patch('zvmsdk.networkops.NetworkOPS._create_invokeScript')
    @mock.patch('zvmsdk.networkops.NetworkOPS._create_network_doscript')
    def test_generate_network_doscript_not_active(self, doscript, invokeScript,
                                    add_file, znetconfig, config, linux_dist):
        net_conf_files = [('target1', 'content1')]
        net_cmd_file = [('target2', 'content2')]
        net_conf_cmds = ''
        clean_cmd = ''
        net_enable = ''
        userid = 'fakeid'
        os_version = 'rhel7.2'
        network_info = []
        first = False
        network_file_path = '/tmp'
        files_and_cmds = net_conf_files, net_conf_cmds, clean_cmd, net_enable
        files_map = []
        files_map.append({'target_path': 'target1',
                        'source_file': "0000"})
        files_map.append({'target_path': 'target2',
                        'source_file': "0001"})
        linux_dist.return_value = dist.rhel7
        config.return_value = files_and_cmds
        znetconfig.return_value = net_cmd_file
        add_file.return_value = None
        invokeScript.return_value = None
        doscript.return_value = 'result1'

        r1, r2 = self.networkops._generate_network_doscript(userid,
                                    os_version, network_info,
                                    network_file_path, first, active=False)
        linux_dist.assert_called_with(os_version)
        config.assert_called_with(network_file_path, network_info,
                                  first, active=False)
        invokeScript.assert_called_with(network_file_path, clean_cmd,
                                        files_map)
        doscript.assert_called_with(network_file_path)

        self.assertEqual(r1, 'result1')
        self.assertEqual(r2, '')

    @mock.patch('zvmsdk.dist.LinuxDistManager.get_linux_dist')
    @mock.patch.object(dist.rhel7, 'create_network_configuration_files')
    @mock.patch.object(dist.rhel7, 'create_active_net_interf_cmd')
    @mock.patch('zvmsdk.networkops.NetworkOPS._create_znetconfig')
    @mock.patch('zvmsdk.networkops.NetworkOPS._add_file')
    @mock.patch('zvmsdk.networkops.NetworkOPS._create_invokeScript')
    @mock.patch('zvmsdk.networkops.NetworkOPS._create_network_doscript')
    def test_generate_network_doscript_active(self, doscript, invokeScript,
                                    add_file, znetconfig, active_cmd,
                                    config, linux_dist):
        net_conf_files = [('target1', 'content1')]
        net_cmd_file = [('target2', 'content2')]
        active_net_cmd = 'create_active_net_interf_cmd'
        net_conf_cmds = ''
        clean_cmd = ''
        net_enable = ''
        userid = 'fakeid'
        os_version = 'rhel7.2'
        network_info = []
        first = False
        network_file_path = '/tmp'
        files_and_cmds = net_conf_files, net_conf_cmds, clean_cmd, net_enable
        files_map = []
        files_map.append({'target_path': 'target1',
                        'source_file': "0000"})
        files_map.append({'target_path': 'target2',
                        'source_file': "0001"})
        linux_dist.return_value = dist.rhel7
        config.return_value = files_and_cmds
        active_cmd.return_value = active_net_cmd
        znetconfig.return_value = net_cmd_file
        add_file.return_value = None
        invokeScript.return_value = None
        doscript.return_value = 'result1'

        r1, r2 = self.networkops._generate_network_doscript(userid,
                                    os_version, network_info,
                                    network_file_path, first, active=True)
        linux_dist.assert_called_with(os_version)
        config.assert_called_with(network_file_path, network_info,
                                  first, active=True)
        invokeScript.assert_called_with(network_file_path, clean_cmd,
                                        files_map)
        doscript.assert_called_with(network_file_path)

        self.assertEqual(r1, 'result1')
        self.assertEqual(r2, active_net_cmd)

    @mock.patch('zvmsdk.smutclient.SMUTClient.query_vswitch')
    def test_vswitch_query(self, query_vswitch):
        self.networkops.vswitch_query("vswitch_name")
        query_vswitch.assert_called_with("vswitch_name")

    @mock.patch.object(shutil, 'rmtree')
    @mock.patch('zvmsdk.smutclient.SMUTClient.execute_cmd')
    @mock.patch('zvmsdk.smutclient.SMUTClient.punch_file')
    @mock.patch('zvmsdk.networkops.NetworkOPS._add_file')
    @mock.patch('zvmsdk.networkops.NetworkOPS._create_znetconfig')
    @mock.patch.object(dist.rhel7, 'get_network_configuration_files')
    @mock.patch.object(dist.rhel7, 'delete_vdev_info')
    @mock.patch.object(dist.rhel7, 'create_active_net_interf_cmd')
    @mock.patch('zvmsdk.dist.LinuxDistManager.get_linux_dist')
    @mock.patch('zvmsdk.smutclient.SMUTClient.get_guest_temp_path')
    def test_delete_network_configuration(self, temp_path, linux_dist,
                                          active_cmd, delete_vdev,
                                          get_netconf_files, znetconfig,
                                          add_file, punch, execute_cmd,
                                          rmtree):
        userid = 'fakeid'
        os_version = 'rhel7.2'
        vdev = '1000'

        net_cmd_file = [('target', 'content')]
        active_net_cmd = 'create_active_net_interf_cmd'
        delete_vdev_info = 'delete_vdev_info'
        get_network_configuration_files = 'network_conf_file'
        network_file_path = '/tmp'

        temp_path.return_value = network_file_path
        linux_dist.return_value = dist.rhel7
        active_cmd.return_value = active_net_cmd
        delete_vdev.return_value = delete_vdev_info
        get_netconf_files.return_value = get_network_configuration_files
        znetconfig.return_value = net_cmd_file
        add_file.return_value = None
        rmtree.return_value = None

        self.networkops.delete_network_configuration(userid, os_version, vdev,
                              active=True)
        temp_path.assert_called_with(userid)
        linux_dist.assert_called_with(os_version)
        get_netconf_files.assert_called_with(vdev)
        delete_vdev.assert_called_with(vdev)
        punch.assert_called_with(userid, '/tmp/DEL1000.sh', "X")
        execute_cmd.assert_called_with(userid, active_net_cmd)

    @mock.patch('zvmsdk.smutclient.SMUTClient.dedicate_OSA')
    def test_dedicate_OSA(self, dedicate_OSA):
        self.networkops.dedicate_OSA("fakeid", 'F000', vdev='1000',
                                     active=True)
        dedicate_OSA.assert_called_with("fakeid", 'F000', vdev='1000',
                                        active=True)



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Handler for the image of the sdk API."""
import json

from zvmconnector import connector
from zvmsdk import config
from zvmsdk import log
from zvmsdk import utils
from zvmsdk.sdkwsgi.handlers import tokens
from zvmsdk.sdkwsgi.schemas import image
from zvmsdk.sdkwsgi import util
from zvmsdk.sdkwsgi import validation


_IMAGEACTION = None
CONF = config.CONF
LOG = log.LOG


class ImageAction(object):

    def __init__(self):
        self.client = connector.ZVMConnector(connection_type='socket',
                                             ip_addr=CONF.sdkserver.bind_addr,
                                             port=CONF.sdkserver.bind_port)

    @validation.schema(image.create)
    def create(self, body):
        image = body['image']
        image_name = image['image_name']
        url = image['url']
        remote_host = image.get('remote_host', None)
        image_meta = image['image_meta']

        info = self.client.send_request('image_import', image_name,
                                        url, image_meta, remote_host)
        return info

    @validation.query_schema(image.query)
    def get_root_disk_size(self, req, name):
        # FIXME: this param need combined image nameg, e.g the profile
        # name, not the given image name from customer side
        info = self.client.send_request('image_get_root_disk_size',
                                        name)
        return info

    def delete(self, name):
        info = self.client.send_request('image_delete', name)
        return info

    @validation.query_schema(image.query)
    def query(self, req, name):
        info = self.client.send_request('image_query', name)
        return info

    @validation.schema(image.export)
    def export(self, name, body):
        location = body['location']
        dest_url = location['dest_url']
        remotehost = location.get('remote_host', None)
        info = self.client.send_request('image_export', name,
                                        dest_url, remotehost)
        return info


def get_action():
    global _IMAGEACTION
    if _IMAGEACTION is None:
        _IMAGEACTION = ImageAction()
    return _IMAGEACTION


@util.SdkWsgify
@tokens.validate
def image_create(req):

    def _image_create(req):
        action = get_action()
        body = util.extract_json(req.body)
        return action.create(body=body)

    info = _image_create(req)

    info_json = json.dumps(info)
    req.response.body = utils.to_utf8(info_json)
    req.response.status = util.get_http_code_from_sdk_return(info,
        additional_handler = util.handle_already_exists)
    req.response.content_type = 'application/json'
    return req.response


@util.SdkWsgify
@tokens.validate
def image_get_root_disk_size(req):

    def _image_get_root_disk_size(req, name):
        action = get_action()
        return action.get_root_disk_size(req, name)

    name = util.wsgi_path_item(req.environ, 'name')
    info = _image_get_root_disk_size(req, name)

    info_json = json.dumps(info)
    req.response.body = utils.to_utf8(info_json)
    req.response.content_type = 'application/json'
    req.response.status = util.get_http_code_from_sdk_return(info)
    return req.response


@util.SdkWsgify
@tokens.validate
def image_delete(req):

    def _image_delete(name):
        action = get_action()
        return action.delete(name)

    name = util.wsgi_path_item(req.environ, 'name')
    info = _image_delete(name)

    info_json = json.dumps(info)
    req.response.body = utils.to_utf8(info_json)
    req.response.status = util.get_http_code_from_sdk_return(info, default=200)
    req.response.content_type = 'application/json'
    return req.response


@util.SdkWsgify
@tokens.validate
def image_export(req):

    def _image_export(name, req):
        action = get_action()
        body = util.extract_json(req.body)
        return action.export(name, body=body)

    name = util.wsgi_path_item(req.environ, 'name')
    info = _image_export(name, req)

    info_json = json.dumps(info)
    req.response.body = utils.to_utf8(info_json)
    req.response.status = util.get_http_code_from_sdk_return(info,
        additional_handler=util.handle_not_found)
    req.response.content_type = 'application/json'
    return req.response


@util.SdkWsgify
@tokens.validate
def image_query(req):

    def _image_query(imagename, req):
        action = get_action()
        return action.query(req, imagename)

    imagename = None
    if 'imagename' in req.GET:
        imagename = req.GET['imagename']
    info = _image_query(imagename, req)

    info_json = json.dumps(info)
    req.response.body = utils.to_utf8(info_json)
    req.response.status = util.get_http_code_from_sdk_return(info,
        additional_handler = util.handle_not_found)
    req.response.content_type = 'application/json'
    return req.response
